#ifndef shell_h
#define shell_h

#include <string.h>
#include <stdlib.h>
#include "comparse.h"
#include <fcntl.h>
char* path;

void argprint(char *arg[])
{
  int index = 0;
  while (arg[index] != NULL)
    {
      printf("argument[%d]: [%s] ", index, arg[index]);
      index++;
    }
  printf("\n");
  return;
}

void mrdln(char **buf,FILE *fin)
{
    char ch=0;
    if ((*buf)!=NULL) free((*buf));
    *buf=NULL;
    while ((ch!='\n'))
      {
        ch=fgetc(fin);
        if ((ch!=13) && (ch!=10)) addchar(buf,ch);    
      }
    if (buf==NULL)
      addchar(buf, 0);
}

void changepath(char *dir)
{
  int idx;
  int done;
  char *pwd, *pwd1;
  printf("%s\n",dir);
  if (dir==NULL)
    {
    }
  else if (strcmp(dir,".")==0)
    return;
  else if(strcmp(dir,"..")==0)
    {
      done=0;
      pwd1=getenv("PWD");
      pwd=(char *)malloc(strlen(pwd1)+1);
      strcpy(pwd,pwd1);
      for(idx=strlen(pwd);idx>=0;idx--)
	{
	  if ((done==0) && (pwd[idx]=='/'))
	    {
	      if (idx==0)
		pwd[idx+1]=0;
	      else
		pwd[idx]=0;
	      done=1;
	    }
	}
      if (chdir(pwd)!=0)
	printf("Cannot change to directory %s\n",pwd);
      else
	setenv("PWD",pwd,1);
    }
  else if(dir[0]=='/')
    {
      pwd=(char *)malloc(strlen(dir)+1);
      strcpy(pwd,dir);
      if (chdir(pwd)!=0)
	printf("Cannot change to directory %s\n",pwd);
      else
	setenv("PWD",pwd,1);
    }
  else
    {
      pwd1=getenv("PWD");
      pwd=(char *)malloc(strlen(pwd1)+2+strlen(dir));
      strcpy(pwd,pwd1);
      if (pwd1[strlen(pwd1)]!='/')
	strcat(pwd,"/");
      strcat(pwd,dir);
      if (chdir(pwd)!=0)
	printf("Cannot change to directory %s\n",pwd);
      else
	setenv("PWD",pwd,1);
    }
}

void create(char *program, char *args [], int backgnd, int in, int out, int pipeIO[][2], int pipeOP, int errout)
{
  char *token, *run, *pwdpath, *temppath;
  int ret = -1, cid, id;
  printf("program [%s] background(%d)\n", program, backgnd);
  if(strcasecmp(program,"cd")==0)
    {
      changepath(args[1]);
      return;
    }
  else if (program[0] == '/')
    {
      run = (char*)malloc(strlen(program));
      strcpy(run, program);
      ret=access(run, X_OK|F_OK);
    }
  else
    {
      if (program[0] == '.')
	{
	  pwdpath = getenv("PWD");
	  run = (char*)malloc(strlen(program) + strlen(pwdpath) + 1);
	  strcpy(run, pwdpath);
	  strcat(run, "/");
	  strcat(run, program);
	  ret=access(run, X_OK|F_OK);
	}
      else
	{
	  temppath = (char*)malloc(strlen(path));
	  strcpy(temppath, path);
	  token=strtok(temppath, ":");
	  while ((token != NULL) && (ret != 0))
	    {
	      run = (char*)malloc(strlen(program) + strlen(token)+ 2);
	      strcpy(run, token);
	      strcat(run, "/");
	      strcat(run, program);
	      printf("%s (%d)", run, backgnd);
	      ret=access(run, X_OK|F_OK);
	      printf(" ret: %d, token: %s\n", ret, token);
	      token=strtok(NULL, ":");
	      if ((ret != 0) && (token != NULL))
		{
		free(run);
		run=NULL;
		}
	    }
	}
    }
  if (ret==0)
    {
      cid=fork();
      if (cid==0)
	{
	  switch(pipeOP)
	    {
	    case 0:
	      close(1);
	      dup(pipeIO[0][1]);
	      close(pipeIO[0][1]);
	      close(pipeIO[0][0]);
	      break;
	    case 1:
	      close(1);
	      dup(pipeIO[1][1]);
	      close(pipeIO[1][1]);
	      close(0);
	      dup(pipeIO[0][0]);
	      close(pipeIO[0][0]);
	      close(pipeIO[0][1]);
	      close(pipeIO[1][0]);
	      break;
	    case 2:
	      close(0);
	      dup(pipeIO[1][0]);
	      close(pipeIO[1][0]);
	      close(pipeIO[1][1]);
	      break;
	    }
	  if (in != 0)
	    {
	      close(0);
	      dup(in);
	      close(in);
	    }
	  if (out != 0)
	    {
	      close(1);
	      dup(out);
	      close(out);
	    }
	  if (errout != 0)
	    {
	      close(2);
	      dup(errout);
	      close(errout);
	    }
	  args[0] = program;
	  execvp(run, args);
	}
      else
	{
	  if(backgnd != 1)
	    {
	      id = 0;
	      while (id != cid)
		{
		  id = wait();	    
		}
	    }
	}
    }
  else
    {
      printf("bmsh: %s: command not found\n",run);
    }
  if (run!=NULL) free(run);
  return;
}

int process(char *command)
{
  char *tok,*com,op,*exec, *dash;
  int state = 0, fidin = 0, fidout = 0, fiderr = 0, backgnd = 0;
  int pipeID[2],pipeIO[2][2], pipeOP, dmem = 0;
  char *args[10];
  int index, argidx=0;
  for (index = 0; index < 10; index++)
    {
      args[index] = NULL;
    }

  backgnd=0;
  pipeIO[0][0]=0;
  pipeIO[0][1]=0;
  pipeIO[1][0]=0;
  pipeIO[1][1]=0;
  pipeOP=3;
  /*com=strxspc(command);*/
  tok=mstrtok(command," <&|->",&op);
  exec = tok;
  if (tok==NULL)
    return 0;
  
  while (tok!=NULL)
    {
      printf("exec: %s\n",exec);
      if (strcmp(exec, "")==0)
	{
	  exec = tok;
	}
      if (strcasecmp(exec,"quit")==0)
	{
	  return 1;
	}
      switch (state)
	{
	case 0: 
	  switch (op)
	    {
	    case '>':
	      if (strcmp(tok,"1")==0)
		state = 1;
	      else if(strcmp(tok,"2")==0)
		state = 2;
	      break;
	    case '<':
	      if (strcmp(tok, " ")!=0)
		args[argidx++] = tok;
	      state = 3;
	      break;
	    case '&':
	      if (strcmp(tok, " ")!=0)
		args[argidx++] = tok;
	      backgnd = 1;
	      break;
	    case '|':
	      backgnd = 1;
	      if (pipeOP==3)
		{
		  pipe(pipeID);
		  pipeIO[0][0]=pipeID[0];
		  pipeIO[0][1]=pipeID[1];
		  pipeOP=0;
		  fidin=0;
		  fidout=0;
		  fiderr=0;
		}
	      else if (pipeOP==0)
		{
		  pipeOP=1;
		  pipeIO[0][0]=pipeID[0];
		  pipeIO[0][1]=pipeID[1];
		  pipe(pipeID);
		  pipeIO[1][0]=pipeID[0];
		  pipeIO[1][1]=pipeID[1];
		  fidin=0;
		  fidout=0;
		  fiderr=0;
		}
	      else if (pipeOP==1)
		{
		      close(pipeIO[0][0]);
		      close(pipeIO[0][1]);
		      pipeIO[0][0]=pipeID[0];
		      pipeIO[0][1]=pipeID[1];
		      pipe(pipeID);
		      pipeIO[1][0]=pipeID[0];
		      pipeIO[1][1]=pipeID[1];
		      fidin=0;
		      fidout=0;
		      fiderr=0;
		}
	      /*printf("pipe: %d %d\n", pipeID[0], pipeID[1]);
	      pipe(pipeID);
	      printf("pipe: %d %d\n", pipeID[0], pipeID[1]);
	      /*fidout = pipeID[1];*/      
	      if ((strcmp(tok,exec)!=0) && (strcmp(tok, " ")!=0))
		args[argidx++] = tok;
	      create(exec, args, backgnd, fidin, fidout, pipeIO, pipeOP, fiderr);
	      for (index = 0; index < 10; index++)
		{
		  args[index] = NULL;
		}
	      backgnd = 0;	      
	      /*fidin = pipeID[0];*/
	      fidout = 0;
	      fiderr = 0;
	      argidx = 0;
	      state = 4;
	      break;
	    case '-':
	      state = 5;
	      break;
	    case ' ':
	      if (strcmp(tok, "") != 0)
		args[argidx++]=tok;
	      break;
	    default:
	      if (strcmp(tok, "") != 0)
		args[argidx++]=tok;
	      break;
	    }
	  break;
      	case 1:
	  printf("state 1\n");
	  if (strcmp(tok, "") != 0)
	    {
	      printf("tok: %s", tok);
	      fidout = open(tok, O_WRONLY|O_CREAT);
	      state = 0;
	    }	 
	  break;
	case 2:
	  if (strcmp(tok, "") != 0)
	    {
	      fiderr = open(tok, O_WRONLY|O_CREAT);
	      state = 0;
	    }
	  break;
	case 3:
	  if (strcmp(tok, "") != 0)
	    {
	      fidin = open(tok, O_RDONLY);
	      state = 0;
	    }
	  break;
	case 4: 
	  if (strcmp(tok, "") != 0)
	    {
	      exec = tok;
	      state = 0;
	    }
	  break;
	case 5:
	  dmem = 1;
	  dash = (char*)malloc(sizeof(tok)+1);
	  strcpy(dash, tok);
	  addcharbeg(&dash, '-');
	  printf("dash: %s\n", dash);
	  args[argidx++]=dash;
	  state = 0;
	  break;
	}
      tok=mstrtok(NULL," <&|->",&op);
      printf("op: [%c] state:%d tok: %s pipe: %d \n", op, state, tok,pipeOP);
    }      
  if (pipeOP==0)
    {
      pipeIO[1][0]=pipeID[0];
      pipeIO[1][1]=pipeID[1]; 
      pipeOP=2;
    }
  if (pipeOP==1)
    {
      close(pipeIO[0][0]);
      close(pipeIO[0][1]);
      pipeIO[1][0]=pipeID[0];
      pipeIO[1][1]=pipeID[1]; 
      pipeOP=2;
    }
  args[argidx++] = tok;
  printf("exe: %s OP: %d",exec,pipeOP);
  argprint(args);
  create(exec, args, backgnd, fidin, fidout, pipeIO, pipeOP, fiderr);
  if (pipeOP==2)
    {
      close(pipeIO[1][0]);
      close(pipeIO[1][1]);
    }
  if (dmem == 1)
    free(dash);
  return 0;
}

#endif
